@@ -1,5 +1,5 @@
 """
- * $Id: 1.1+3 utils.py e48fa1383c14 2021-05-19 od $
+ * $Id: 1.1+4 utils.py d9a751b54766 2021-05-28 od $
  *
  * This file is part of the Cloud Services Integration Platform (CSIP),
  * a Model-as-a-Service framework, API and application suite.
@@ -9,7 +9,7 @@
  * OMSLab licenses this file to you under the MIT license.
  * See the LICENSE file in the project root for more information.
 """
-import glob
+
 from os import path
 from pathlib import Path
 from shutil import make_archive
@@ -17,12 +17,7 @@ from time import sleep
 from typing import List, Dict, Tuple
 from requests.exceptions import Timeout
 
-import json
-import os
-import re
-import requests
-import sys
-
+import json, os, re, requests, sys, time, glob
 
 class Client(object):
     """
@@ -71,10 +66,11 @@ class Client(object):
     HDR_CSIP_REQUESTFILE = "X-CSIP-Request-File"
 
     # CONFIG defaults
-    GET_TIMEOUT = (2,10) # (connect, read)
-    POST_TIMEOUT = (2,30) # (connect, read)
-    RETRY = 3
-    ALLOW_REDIRECTS = True
+    CONN_TIMEOUT = 2  # for "conn_timeout"
+    READ_TIMEOUT = 10 # for "read_timeout"
+    RESP_TIMEOUT = 30 # for "resp_timeout"
+    RETRY = 3  # for "retry"
+    ALLOW_REDIRECTS = True  # for "allow_redirects"
 
     def __init__(self, data: Dict = None, metainfo: Dict = None,
                  parent: 'Client' = None, url: str = None, name: str = '',
@@ -150,9 +146,10 @@ class Client(object):
     def __get0(url: str, conf: Dict):
         """HTTP GET to generate a Client"""
         conf = conf or {}
-        timeout = conf.get('get_timeout', Client.GET_TIMEOUT)
+        timeout = (conf.get('conn_timeout', Client.CONN_TIMEOUT), conf.get('read_timeout', Client.READ_TIMEOUT))
         retry = conf.get('retry', Client.RETRY)
         allow_redirects = conf.get('allow_redirects', Client.ALLOW_REDIRECTS)
+
         http_status = 200
         response = None
         while retry > 0:
@@ -181,12 +178,17 @@ class Client(object):
         return response, http_status
 
     @staticmethod
-    def __post0(url: str, files: Dict, headers: Dict, conf: Dict):
+    def __post0(url: str, files: Dict, headers: Dict, conf: Dict, sync: bool = True):
         """HTTP POST to generate a Client"""
         conf = conf or {}
-        timeout = conf.get('post_timeout', Client.POST_TIMEOUT)
+        if sync:
+            timeout = (conf.get('conn_timeout', Client.CONN_TIMEOUT), conf.get('resp_timeout', Client.RESP_TIMEOUT))
+        else:
+            timeout = (conf.get('conn_timeout', Client.CONN_TIMEOUT), conf.get('read_timeout', Client.READ_TIMEOUT))
+
         retry = conf.get('retry', Client.RETRY)
         allow_redirects = conf.get('allow_redirects', Client.ALLOW_REDIRECTS)
+
         http_status = 200
         response_json = None
         while retry > 0:
@@ -337,10 +339,11 @@ class Client(object):
             url = self.get_url()
             if url is None:
                 raise Exception('No url provided.')
+
         # print(multipart)
         # response = requests.post(url, files=multipart, allow_redirects=True, headers=headers)
 
-        response_json, http_status = Client.__post0(url, multipart, headers, conf)
+        response_json, http_status = Client.__post0(url, multipart, headers, conf, sync)
 
         if http_status == 200:
             data = response_json.get(Client.RESULT, {})
@@ -378,8 +381,14 @@ class Client(object):
 
         callback = callback or cb
 
+        # calc expiration time
+        exp = int(time.time() * 1000.0) + int(conf.get('resp_timeout', Client.RESP_TIMEOUT) * 1000.0)
+
         # -> Submitted
         resp = self.execute(url, files=files, sync=False, headers=headers, conf=conf)
+        if exp < int(time.time() * 1000.0):
+            return Client(None, {}, http_status=-1)
+
         callback(resp, "")
 
         fp = (first_poll or resp.get_first_poll()) / 1000
@@ -395,6 +404,9 @@ class Client(object):
 
         # Submitted -> Running
         r = resp.query_execute(conf=conf)
+        if exp < int(time.time() * 1000.0):
+            return Client(None, {}, http_status=-1)
+
         progress = r.get_progress()
         callback(r, progress)
         while r.get_status() == Client.STATUS_RUNNING \
@@ -402,6 +414,8 @@ class Client(object):
             sleep(np)
             # Running -> Running | Finished | Failed
             r = resp.query_execute(conf=conf)
+            if exp < int(time.time() * 1000.0):
+                return Client(None, {}, http_status=-1)
             progress = r.get_progress()
             callback(r, progress)
         return r
